/**
 * JASMINe
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: PackageAdderListener.java 1741 2008-07-21 15:13:28Z jlegrand $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.rules.osgi.impl;

import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceEvent;
import org.osgi.framework.ServiceListener;
import org.osgi.framework.ServiceReference;
import org.ow2.jasmine.rules.osgi.api.IOSGiDroolsWorkingMemory;
import org.ow2.jasmine.rules.osgi.api.IPackageAdder;

/**
 * Listener that add rules package into the working memory when an instance of service {@link IPackageAdder} is registered.
 * This occurs when a bundle, to which its activator implements {@link RulesActivator}, is started.
 * @author Loris Bouzonnet
 */
public class PackageAdderListener implements ServiceListener {

    /**
     * Working memory to add rules package.
     */
    private final IOSGiDroolsWorkingMemory droolsWorkingMemory;

    /**
     * OSGi bundle context to retrieve the incoming {@link IPackageAdder}.
     */
    private final BundleContext context;

    /**
     * Construct a new listener for the given context and working memory.
     * @param context OSGi bundle context
     * @param droolsWorkingMemory working memory to add rules package
     */
    public PackageAdderListener(
            final BundleContext context,
            final IOSGiDroolsWorkingMemory droolsWorkingMemory) {
        this.context = context;
        this.droolsWorkingMemory = droolsWorkingMemory;
    }

    public void serviceChanged(final ServiceEvent event) {
        ServiceReference ref = event.getServiceReference();
        // React on REGISTERED|UNREGISTERING events
        switch(event.getType()) {
            case ServiceEvent.REGISTERED:
                IPackageAdder packageAdder = (IPackageAdder) context.getService(ref);
                try {
                    droolsWorkingMemory.addPackage(ref, packageAdder);
                } catch (Exception e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
                break;
            case ServiceEvent.UNREGISTERING:
                try {
                    droolsWorkingMemory.removePackage(ref);
                } catch (Exception e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
        }

    }

}
